@@ -50,6 +50,15 @@ class PublicArgumentError(argparse.ArgumentError):
     """
 
 
+class PublicArgumentTypeError(argparse.ArgumentTypeError):
+    """
+    Argument type error with public message. Exceptions of this type raised under
+    `prefix_stack_trace` or `print_prefixed_stack_trace_and_raise` will have
+    the message prefixed with `PREFIX` in both the printed stack trace and the
+    re-raised exception.
+    """
+
+
 class PublicKeyError(KeyError):
     """
     Key error with public message. Exceptions of this type raised under
@@ -108,6 +117,7 @@ default_allow_list = [
     PublicValueError.__name__,
     PublicRuntimeError.__name__,
     PublicArgumentError.__name__,
+    PublicArgumentTypeError.__name__,
     PublicKeyError.__name__,
     PublicTypeError.__name__,
     PublicIndexError.__name__,
