@@ -9,3 +9,5 @@ and submit AML pipelines
 
 from .pipeline_helper import AMLPipelineHelper  # noqa: F401
 from .federated_learning import FederatedPipelineBase, StepOutput  # noqa: F401
+from .ray_actor import ray_actor_on_shrike, b64_decode, b64_encode  # noqa: F401
+from .argparser_utils import get_args_from_component_spec  # noqa: F401
