@@ -7,10 +7,13 @@ Pipeline helper class to create pipelines loading modules from a flexible manife
 
 
 from azure.ml.component import Component
+from azure.ml.component._core._component_definition import ComponentDefinition
 from dataclasses import dataclass, field
 import os
 import logging
 from typing import Optional, List, Tuple
+from packaging.specifiers import SpecifierSet
+from packaging.version import Version, InvalidVersion
 
 from shrike.pipeline.aml_connect import current_workspace
 
@@ -256,6 +259,11 @@ class AMLModuleLoader:
         for registry in self.registries:
             try:
                 log.info(f"Loading component {module_name} from registry {registry}")
+
+                module_version = solve_module_version(
+                    module_name, module_version, registry=registry
+                )
+
                 loaded_module_class = Component.load(
                     current_workspace(),
                     name=module_name,
@@ -271,9 +279,16 @@ class AMLModuleLoader:
         for feed in self.feeds:
             try:
                 log.info(f"Loading component {module_name} from feed {feed}")
+
+                module_name = f"azureml.feed://{feed}/{module_name}"
+
+                module_version = solve_module_version(
+                    module_name, module_version, workspace=current_workspace()
+                )
+
                 loaded_module_class = Component.load(
                     current_workspace(),
-                    name=f"azureml.feed://{feed}/{module_name}",
+                    name=module_name,
                     version=module_version,
                 )
                 self.put_in_cache(module_cache_key, loaded_module_class)
@@ -284,6 +299,10 @@ class AMLModuleLoader:
 
         try:
             # try without namespace first
+            module_version = solve_module_version(
+                module_name, module_version, workspace=current_workspace()
+            )
+
             loaded_module_class = Component.load(
                 current_workspace(),
                 name=module_name,
@@ -299,6 +318,10 @@ class AMLModuleLoader:
                 f"    Trying to load module {module_name} with namespace {module_namespace}."
             )
             module_name = module_namespace + "://" + module_name
+            module_version = solve_module_version(
+                module_name, module_version, workspace=current_workspace()
+            )
+
             loaded_module_class = Component.load(
                 current_workspace(),
                 name=module_name,
@@ -407,6 +430,53 @@ class AMLModuleLoader:
         return loaded_modules
 
 
+def solve_module_version(module_name, module_version, workspace=None, registry=None):
+
+    if module_version is None:
+        return module_version
+
+    try:
+        module_version_PEP440 = Version(module_version)
+        if str(module_version_PEP440) != module_version:
+            log.warning(
+                "We suggest adopting PEP440 versioning for your component {module_name}!"
+            )
+
+    except InvalidVersion as e:
+        log.info(f"{module_version} is a version constraint. Try to solve it...")
+
+        spec = SpecifierSet(module_version)
+        if registry:
+            components = ComponentDefinition.list(
+                registry_name=registry, name=module_name
+            )
+        elif workspace:
+            components = ComponentDefinition.list(workspace=workspace, name=module_name)
+
+        versions = []
+        for component in components:
+            version = component.version
+            version_PEP440 = Version(version)
+            if str(version_PEP440) != version:
+                log.warning(
+                    f"Version {version} does not follow PEP440 versioning, skipping ..."
+                )
+            else:
+                versions.append(version_PEP440)
+
+        compatible_versions = list(spec.filter(versions))
+        if compatible_versions:
+            module_version = max(compatible_versions)
+            log.info(f"Solved version for {module_name} is {module_version}.")
+
+        else:
+            raise ValueError(
+                f"No version exists for the constraint {module_version}. Existing versions: {versions}"
+            )
+
+    return str(module_version)
+
+
 def _check_use_local_syntax_valid(use_local_list) -> bool:
     use_local_except_for = True if use_local_list[0].startswith("!") else False
     if use_local_except_for:
