@@ -4,7 +4,7 @@
 """ Configuration dataclasses for AMLPipelineHelper """
 from dataclasses import dataclass, field
 from omegaconf import MISSING
-from typing import Optional, Any, Dict, List, Union
+from typing import Optional, Any, Dict, List
 from shrike.pipeline.module_helper import module_loader_config, module_manifest
 
 # Default config for HDI components
@@ -38,6 +38,7 @@ class pipeline_cli_config:  # pylint: disable=invalid-name
     - `endpoint_description` (Optional[str]): description of the endpoint in which to publish the pipeline
     - `log_error_only` （bool): set to True for logging error only (default to False)
     - `skip_validation` (bool): set to True to not run pipeline.validate() after building and submit with "skip_validation=True" (default to False)
+    - `script` (Optional[str]): for Ray actor orchestration only, the script containing classes to be wrapped as Ray actors
     """
 
     regenerate_outputs: bool = False
@@ -61,6 +62,7 @@ class pipeline_cli_config:  # pylint: disable=invalid-name
     endpoint_description: Optional[str] = None
     log_error_only: bool = False
     skip_validation: bool = False
+    script: Optional[str] = None
 
 
 @dataclass
@@ -104,6 +106,7 @@ class pipeline_compute_config:  # pylint: disable=invalid-name
     - `windows_output_mode` (str): output mode for windows compute targets (default to "upload"), more details at https://componentsdk.azurewebsites.net/concepts/inputs-and-outputs.html#output-dataset-mode
     - `hdi_prod_target` (str): name of HDI regular compute target
     - `hdi_dc_target` (Optional[str]): name of HDI detonation chamber compute target
+    - `synapse_prod_target` (str): name of Synapse regular compute target
     - `hdi_driver_memory` (str): HDI driver memory in GB (default to "4g")
     - `hdi_driver_cores` (int): number of HDI driver cores (default to 2)
     - `hdi_executor_memory` (str): HDI executor memory in GB (default to "3g")
@@ -137,6 +140,7 @@ class pipeline_compute_config:  # pylint: disable=invalid-name
 
     hdi_prod_target: str = MISSING
     hdi_dc_target: Optional[str] = None
+    synapse_prod_target: Optional[str] = None
     hdi_driver_memory: str = "4g"
     hdi_driver_cores: int = 2
     hdi_executor_memory: str = "3g"
@@ -208,6 +212,7 @@ class federated_config:
     - `config_group` (Optional[Dict[str, Any]]): config applying to all or some silos; `default_config` will be applied to all silos, and you can also define any customized config
     - `data_transfer_component` (str): name of the data transfer component **registered** in your workspace
     - `deactivate_data_transfer` (bool): set to True to not move the data between central datastore and silo datastore, so that previous results can be reused (default to False)
+    - `use_secure_aggregation` (bool): set to True when users want to use secure aggation in their federated learning pipeline (default to False)
     """
 
     silos: Dict[str, silo] = MISSING
@@ -216,6 +221,7 @@ class federated_config:
     config_group: Optional[Dict[str, Any]] = MISSING
     data_transfer_component: str = MISSING
     deactivate_data_transfer: bool = False
+    use_secure_aggregation: bool = False
 
 
 def default_config_dict():
