@@ -19,6 +19,7 @@ import jsonpath_ng
 from typing import Callable
 from toposort import toposort_flatten
 import datetime
+from dataclasses import dataclass
 from functools import lru_cache, wraps
 
 try:
@@ -30,6 +31,7 @@ try:
 
     import azureml
     from azureml.core import Datastore
+    from azureml.core import Run
     from azureml.core import Experiment
     from azureml.core import Dataset
     from azureml.pipeline.core import PipelineRun
@@ -44,14 +46,23 @@ from shrike import __version__
 from shrike.pipeline.aml_connect import azureml_connect, current_workspace
 from shrike.pipeline.canary_helper import get_repo_info
 from shrike.pipeline.module_helper import AMLModuleLoader
-from shrike.pipeline.pipeline_config import default_config_dict, HDI_DEFAULT_CONF
+from shrike.pipeline.pipeline_config import (
+    default_config_dict,
+    HDI_DEFAULT_CONF,
+)
 from shrike.pipeline.telemetry_utils import TelemetryLogger
-from shrike._core import is_eyesoff_helper
+from shrike._core import is_eyesoff_helper, POLYMER_FEED
 
 
 log = logging.getLogger(__name__)
 
 
+@dataclass
+class Data:
+    datastore_name: str
+    path: str
+
+
 class AMLPipelineHelper:
     """Helper class for building pipelines"""
 
@@ -429,6 +440,8 @@ class AMLPipelineHelper:
                 if target is not None
                 else self.config.compute.windows_cpu_dc_target
             )
+            if self.config.compute.hdi_dc_target:
+                datastore_name = self.config.compute.dc_datastore
         else:
             target = (
                 target
@@ -464,10 +477,14 @@ class AMLPipelineHelper:
             )
 
         if output_mode:
-            self._set_all_outputs_to(module_instance, output_mode)
+            self._set_all_outputs_to(
+                module_instance, output_mode, datastore_name=datastore_name
+            )
         else:
             self._set_all_outputs_to(
-                module_instance, self.config.compute.windows_output_mode
+                module_instance,
+                self.config.compute.windows_output_mode,
+                datastore_name=datastore_name,
             )
 
     def _apply_hdi_runsettings(
@@ -508,6 +525,7 @@ class AMLPipelineHelper:
                 raise Exception(
                     f"Your HDI component {module_name} is using local version. Please specify hdi_dc_target"
                 )
+            datastore_name = self.config.compute.dc_datastore
         else:
             target = (
                 target if target is not None else self.config.compute.hdi_prod_target
@@ -563,6 +581,96 @@ class AMLPipelineHelper:
             module_instance, output_mode, datastore_name=datastore_name
         )
 
+    def _apply_synapse_runsettings(
+        self,
+        module_name,
+        module_instance,
+        target=None,
+        driver_cores=None,
+        driver_memory=None,
+        executor_memory=None,
+        executor_cores=None,
+        number_executors=None,
+        conf=None,
+        input_mode=None,
+        output_mode=None,
+        datastore_name=None,
+        **custom_runtime_arguments,
+    ):
+        """Applies settings for a Synapse component. Will use same configs as for HDI components.
+
+        Args:
+            module_name (str): name of the module from the module manifest (required_modules() method)
+            module_instance (Module): the AML module we need to add settings to
+            target (str): force target compute over hydra conf
+            driver_cores (int): force driver_cores over hydra conf
+            driver_memory (str): force driver_memory over hydra conf
+            executor_memory (int): force executor_memory over hydra conf
+            executor_cores (int): force executor_cores over hydra conf
+            number_executors (int): force number_executors over hydra conf
+            conf (str): force conf over hydra conf
+            input_mode (str): force input_mode over hydra conf
+            output_mode (str): force output_mode over hydra conf
+            custom_runtime_arguments (dict): any additional custom args
+        """
+
+        if self.module_loader.is_local(module_name) and self.is_eyesoff():
+            log.warning(
+                "The job is expected to fail, as local Synapse components are not supported in eyes-off environments"
+            )
+
+        target = (
+            target if target is not None else self.config.compute.synapse_prod_target
+        )
+        log.info(
+            f"Using Synapse compute target {target} to run {module_name} from pipeline class {self.__class__.__name__}"
+        )
+        if custom_runtime_arguments:
+            log.info(f"Adding custom runtime arguments {custom_runtime_arguments}")
+
+        new_conf = (
+            self.config.compute.hdi_conf if "hdi_conf" in self.config.compute else None
+        )
+        if conf is not None:
+            new_conf = conf
+        if new_conf is not None:
+            if isinstance(new_conf, str):
+                new_conf = json.loads(new_conf)
+            elif isinstance(new_conf, DictConfig):
+                new_conf = flatten(dict(new_conf), reducer="dot")
+            else:
+                raise ValueError(
+                    "computed.hdi_conf is not a valid json string or a single tested configuration."
+                )
+
+        module_instance.runsettings.configure(target=target)
+
+        module_instance.runsettings.spark.configure(
+            driver_memory=driver_memory
+            if driver_memory is not None
+            else self.config.compute.hdi_driver_memory,
+            driver_cores=driver_cores
+            if driver_cores is not None
+            else self.config.compute.hdi_driver_cores,
+            executor_memory=executor_memory
+            if executor_memory is not None
+            else self.config.compute.hdi_executor_memory,
+            executor_cores=executor_cores
+            if executor_cores is not None
+            else self.config.compute.hdi_executor_cores,
+            number_executors=number_executors
+            if number_executors is not None
+            else self.config.compute.hdi_number_executors,
+            conf=new_conf,
+            **custom_runtime_arguments,
+        )
+
+        if input_mode:
+            self._set_all_inputs_to(module_instance, input_mode)
+        self._set_all_outputs_to(
+            module_instance, output_mode, datastore_name=datastore_name
+        )
+
     def _apply_parallel_runsettings(
         self,
         module_name,
@@ -615,6 +723,8 @@ class AMLPipelineHelper:
                     _target = self.config.compute.linux_gpu_dc_target
                 else:
                     _target = self.config.compute.linux_cpu_dc_target
+            if self.config.compute.hdi_dc_target:
+                datastore_name = self.config.compute.dc_datastore
         else:
             if windows:
                 if gpu:
@@ -669,10 +779,9 @@ class AMLPipelineHelper:
 
         if input_mode:
             self._set_all_inputs_to(module_instance, input_mode)
-        if output_mode:
-            self._set_all_outputs_to(
-                module_instance, output_mode, datastore_name=datastore_name
-            )
+        self._set_all_outputs_to(
+            module_instance, output_mode, datastore_name=datastore_name
+        )
 
     def _apply_linux_runsettings(
         self,
@@ -708,6 +817,8 @@ class AMLPipelineHelper:
                 if target is not None
                 else self.config.compute.linux_gpu_dc_target
             )
+            if self.config.compute.hdi_dc_target:
+                datastore_name = self.config.compute.dc_datastore
             log.info(
                 f"Using target {target} for local code GPU module {module_name} from pipeline class {self.__class__.__name__}"
             )
@@ -726,6 +837,8 @@ class AMLPipelineHelper:
                 if target is not None
                 else self.config.compute.linux_cpu_dc_target
             )
+            if self.config.compute.hdi_dc_target:
+                datastore_name = self.config.compute.dc_datastore
             log.info(
                 f"Using target {target} for local CPU module {module_name} from pipeline class {self.__class__.__name__}"
             )
@@ -920,6 +1033,8 @@ class AMLPipelineHelper:
                     target = self.config.compute.linux_gpu_dc_target
                 else:
                     target = self.config.compute.linux_cpu_dc_target
+            if self.config.compute.hdi_dc_target:
+                datastore_name = self.config.compute.dc_datastore
         else:
             if windows:
                 if gpu:
@@ -1071,6 +1186,7 @@ class AMLPipelineHelper:
         scope="auto",
         datatransfer="auto",
         sweep="auto",
+        synapse="auto",
         **custom_runtime_arguments,
     ):
         """Applies regular settings for a given component.
@@ -1078,13 +1194,14 @@ class AMLPipelineHelper:
         Args:
             component_instance (Component): the AML component we need to add settings to
             gpu (bool): is the component using GPU?
-            hdi (bool): is the component using HDI/Spark?
+            hdi (bool): is the component using HDI?
             windows (bool): is the component using Windows compute?
             parallel (bool): is the component using ParallelRunStep?
             mpi (bool): is the component using Mpi?
             scope (bool): is the component using scope?
             datatransfer (bool): is the component using datatransfer?
             sweep (bool): is the component using sweep?
+            synapse (bool): is the component using Synapse?
             custom_runtime_arguments (dict): any additional custom args
         """
         # infer component_name
@@ -1100,6 +1217,7 @@ class AMLPipelineHelper:
             scope,
             datatransfer,
             sweep,
+            synapse,
             **custom_runtime_arguments,
         )
 
@@ -1115,6 +1233,7 @@ class AMLPipelineHelper:
         scope="auto",
         datatransfer="auto",
         sweep="auto",
+        synapse="auto",
         **custom_runtime_arguments,
     ):
         """Applies regular settings for a given module.
@@ -1127,12 +1246,21 @@ class AMLPipelineHelper:
             windows (bool): is the module using Windows compute?
             parallel (bool): is the module using ParallelRunStep?
             mpi (bool): is the module using Mpi?
+            scope (bool): is the component using scope?
+            datatransfer (bool): is the component using datatransfer?
+            sweep (bool): is the component using sweep?
+            synapse (bool): is the component using Synapse?
             custom_runtime_arguments (dict): any additional custom args
         """
-        # verifies if module_name corresponds to module_instance
+        # Verifies if module_name corresponds to module_instance
         self._check_module_runsettings_consistency(module_name, module_instance)
 
-        # Auto detect runsettings
+        # Skips if this is a PipelineComponent
+        if str(module_instance.type) == "PipelineComponent":
+            log.info(f"Component {module_name} detected as PipelineComponent.")
+            return
+
+        # Auto detects runsettings
         if hdi == "auto":
             hdi = str(module_instance.type) == "HDInsightComponent"
             if hdi:
@@ -1158,6 +1286,11 @@ class AMLPipelineHelper:
             if sweep:
                 log.info(f"Module {module_name} detected as SweepComponent: {sweep}")
 
+        if synapse == "auto":
+            synapse = str(module_instance.type) == "spark"
+            if synapse:
+                log.info(f"Module {module_name} detected as spark (Synapse): {synapse}")
+
         if windows == "auto":
             if (
                 str(module_instance.type) == "HDInsightComponent"
@@ -1230,6 +1363,12 @@ class AMLPipelineHelper:
             )
             return
 
+        if synapse:
+            self._apply_synapse_runsettings(
+                module_name, module_instance, **custom_runtime_arguments
+            )
+            return
+
         self._apply_linux_runsettings(
             module_name,
             module_instance,
@@ -1400,8 +1539,7 @@ class AMLPipelineHelper:
         old_env_file_path = None
         if env_yaml_override_is_needed:
             spec_dirname = os.path.dirname(spec_path)
-            polymer_pkg_idx = "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/"
-            log.info(f"Will remove {polymer_pkg_idx} from environment.conda.")
+            log.info(f"Will remove {POLYMER_FEED} from environment.conda.")
             try:
                 conda_dependencies_file = spec["environment"]["conda"][
                     "conda_dependencies_file"
@@ -1413,7 +1551,7 @@ class AMLPipelineHelper:
                     new_env_file_path,
                     old_env_file_path,
                 ) = self._remove_polymer_pkg_idx_if_exists_and_save_new(
-                    spec_dirname, conda_dependencies_file, polymer_pkg_idx
+                    spec_dirname, conda_dependencies_file, POLYMER_FEED
                 )
                 if found_index_url:
                     spec["environment"]["conda"]["conda_dependencies_file"] = new_file
@@ -1428,8 +1566,8 @@ class AMLPipelineHelper:
                 for idx, dependency in enumerate(conda_dependencies):
                     if isinstance(dependency, dict) and "pip" in dependency:
                         pip_dependencies = dependency["pip"]
-                        if polymer_pkg_idx in pip_dependencies:
-                            pip_dependencies.remove(polymer_pkg_idx)
+                        if POLYMER_FEED in pip_dependencies:
+                            pip_dependencies.remove(POLYMER_FEED)
                             dependency["pip"] = pip_dependencies
                             conda_dependencies[idx] = dependency
                 spec["environment"]["conda"]["conda_dependencies"][
@@ -1449,7 +1587,7 @@ class AMLPipelineHelper:
                     new_env_file_path,
                     old_env_file_path,
                 ) = self._remove_polymer_pkg_idx_if_exists_and_save_new(
-                    spec_dirname, pip_requirements_file, polymer_pkg_idx
+                    spec_dirname, pip_requirements_file, POLYMER_FEED
                 )
                 if found_index_url:
                     spec["environment"]["conda"]["pip_requirements_file"] = new_file
@@ -1630,7 +1768,7 @@ class AMLPipelineHelper:
         """Connect to the AML workspace using internal config"""
         # Only call azureml_connect if there is not an already a pre-existing workspace
         try:
-            return current_workspace()
+            return self.workspace()
         except:
             return azureml_connect(
                 aml_subscription_id=self.config.aml.subscription_id,
@@ -1653,14 +1791,12 @@ class AMLPipelineHelper:
 
         if not self.config.run.skip_validation:
             log.info("Validating...")
-            pipeline.validate(workspace=current_workspace())
+            pipeline.validate(workspace=self.workspace())
 
         if self.config.run.export:
             log.info(f"Exporting to {self.config.run.export}...")
             with open(self.config.run.export, "w") as export_file:
-                export_file.write(
-                    pipeline._get_graph_json(workspace=current_workspace())
-                )
+                export_file.write(pipeline._get_graph_json(workspace=self.workspace()))
 
         if self.config.run.publish:
             self._publish_to_endpoint(pipeline)
@@ -1674,7 +1810,7 @@ class AMLPipelineHelper:
             # pipeline_run is of the class "azure.ml.component.run", which
             # is different from "azureml.pipeline.core.PipelineRun"
             pipeline_run = pipeline.submit(
-                workspace=current_workspace(),
+                workspace=self.workspace(),
                 experiment_name=self.config.run.experiment_name,
                 description=self.config.run.experiment_description,
                 display_name=self.config.run.display_name,
@@ -1751,9 +1887,7 @@ class AMLPipelineHelper:
                 )
 
             log.info(f"Resuming Experiment {self.config.run.experiment_name}...")
-            experiment = Experiment(
-                current_workspace(), self.config.run.experiment_name
-            )
+            experiment = Experiment(self.workspace(), self.config.run.experiment_name)
             log.info(f"Resuming PipelineRun {self.config.run.pipeline_run_id}...")
             # pipeline_run is of the class "azureml.pipeline.core.PipelineRun"
             pipeline_run = PipelineRun(experiment, self.config.run.pipeline_run_id)
@@ -1824,6 +1958,26 @@ Follow link below to access your pipeline run directly:
                 log.info("*** PIPELINE FINISHED, TESTING WITH canary() METHOD ***")
                 self.canary(self.config, pipeline_run.experiment, pipeline_run)
                 log.info("OK")
+                step_runs = pipeline_run.get_steps()
+                for step_run in step_runs:
+                    pass
+                try:
+                    datasets = []
+                    for output in step_run.get_details()["outputDatasets"]:
+                        dataset = output["dataset"]
+                        dataset_info = dataset._dataflow._steps[0].arguments._pod[
+                            "datastores"
+                        ][0]
+                        print(f"Output dataset {dataset_info}.")
+                        datasets.append(
+                            Data(
+                                datastore_name=dataset_info["datastoreName"],
+                                path=dataset_info["path"],
+                            )
+                        )
+                    return datasets
+                except:
+                    print("no output")
             elif pipeline_run.get_status() == "Failed":
                 log.info("*** PIPELINE FAILED ***")
                 raise Exception("Pipeline failed.")
